@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -41,12 +41,8 @@ $fields = [
 	'triggerid' =>	[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		PAGE_TYPE_HTML.'=='.$page['type']],
 	'eventid' =>	[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		PAGE_TYPE_HTML.'=='.$page['type']],
 	// Ajax
-	'widget' =>	[T_ZBX_STR, O_OPT, P_ACT,
-		IN('"'.WIDGET_HAT_EVENTACK.'","'.WIDGET_HAT_EVENTACTIONMSGS.'","'.WIDGET_HAT_EVENTACTIONMCMDS.'","'.
-			WIDGET_HAT_EVENTLIST.'"'),
-		null
-	],
-	'state'=>	[T_ZBX_INT, O_OPT, P_ACT, IN('0,1'), null]
+	'widget' =>		[T_ZBX_STR, O_OPT, P_ACT,	IN('"'.WIDGET_HAT_EVENTACTIONS.'","'.WIDGET_HAT_EVENTLIST.'"'), null],
+	'state' =>		[T_ZBX_INT, O_OPT, P_ACT,	IN('0,1'), null]
 ];
 check_fields($fields);
 
@@ -143,10 +139,10 @@ $eventTab = (new CTable())
 			))->setHeader(_('Event details'))
 		]),
 		new CDiv([
-			(new CCollapsibleUiWidget(WIDGET_HAT_EVENTACK,
-					makeEventDetailsActionsTable($actions, $users, $mediatypes, $severity_config)
+			(new CCollapsibleUiWidget(WIDGET_HAT_EVENTACTIONS,
+				makeEventDetailsActionsTable($actions, $users, $mediatypes, $severity_config)
 			))
-				->setExpanded((bool) CProfile::get('web.tr_events.hats.'.WIDGET_HAT_EVENTACK.'.state', true))
+				->setExpanded((bool) CProfile::get('web.tr_events.hats.'.WIDGET_HAT_EVENTACTIONS.'.state', true))
 				->setHeader(_('Actions'), [], false, 'tr_events.php'),
 			(new CCollapsibleUiWidget(WIDGET_HAT_EVENTLIST,
 				make_small_eventlist($event,
